# -*- coding: utf-8 -*-
"""Material used to generate and execute bindings generated by the PyXB package.
"""

# Automatically include all the stuff that's necessary for basic
# running.  There are order dependencies here.
from . import basis
from . import datatypes
from . import facets
from . import content

# Do not include the stuff that's required only for code generation
# noimport generate
